package edu.umich.languageid;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import cc.mallet.classify.constraints.ge.MaxEntGEConstraint;
import cc.mallet.fst.semi_supervised.constraints.GEConstraint;
import cc.mallet.pipe.CharSequence2TokenSequence;
import cc.mallet.pipe.FeatureSequence2FeatureVector;
import cc.mallet.pipe.Input2CharSequence;
import cc.mallet.pipe.Noop;
import cc.mallet.pipe.Pipe;
import cc.mallet.pipe.SerialPipes;
import cc.mallet.pipe.Target2Label;
import cc.mallet.pipe.TokenSequence2FeatureSequence;
import cc.mallet.types.Instance;
import cc.mallet.types.InstanceList;
import edu.umich.languageid.io.TestingDataReader;
import edu.umich.languageid.io.TrainingDataReader;

public class DocumentSplitter
{
	public static void main(String[] args)
	{
		if(args[0].startsWith("-h") || args[0].startsWith("--h"))
		{
			System.out.println("Usage: [maxent|maxent-ge|hmm|crf] [lang_id:lang_sample]+ <file_to_label>");
		}
		
		List<String> labels = new ArrayList<String>();
		Map<String, File> languageSampleFiles = new HashMap<String, File>();
		File fileToLabel = new File("");
		
		String method = args[0];
		
		for(int k=1; k<args.length; ++k)
		{
			String arg = args[k];
			if(arg.contains(":"))
			{
				String[] tokens = arg.split(":");
				String languageCode = tokens[0];
				File languageSample = new File(tokens[1]);
				
				languageSampleFiles.put(languageCode, languageSample);
				labels.add(languageCode);
			}
			else
			{
				fileToLabel = new File(arg);
			}
		}
		
		FeatureConfiguration features = new FeatureConfiguration(true, true, true, true, true, true);
		List<Pipe> pipes = new ArrayList<Pipe>();
		pipes.add(new Input2CharSequence("UTF-8"));
		pipes.add(new CharSequence2TokenSequence("[\\pL\\pM\\p{Nd}\\p{Nl}\\p{Pc}[\\p{InEnclosedAlphanumerics}&&\\p{So}]\\p{Nd}\\-_]+"));
		pipes.add(new TokenSequence2FeatureSequence());
		pipes.add(new Target2Label());
		pipes.add(new FeatureSequence2FeatureVector());
		Pipe pipe = new SerialPipes(pipes);
		
		TrainingDataReader trainingDataReader = new TrainingDataReader(features, 10000, pipe);
		InstanceList trainingData = trainingDataReader.readTrainingData(languageSampleFiles);
		
		TestingDataReader testingDataReader = new TestingDataReader(features, pipe);
		InstanceList testingData = testingDataReader.readTestingData(fileToLabel);
		
		if(method.toLowerCase().equals("hmm"))
		{
			EMHMM emhmm = new EMHMM(labels);
			emhmm.printClassification(trainingData, testingData);
		}
		else if(method.toLowerCase().equals("crf"))
		{
			TestingDataReader testingDataSequenceReader = new TestingDataReader(features, pipe);
			InstanceList testingDataSequence = testingDataSequenceReader.readSequence(fileToLabel);
			
			WordClassification wordClassification = new WordClassification();
			
			Map<String, Double> labelProportions = wordClassification.getLanguageProportions(trainingData, testingData);
			Map<String, Map<String, Integer>> labeledFeatureCounts = trainingDataReader.labeledFeatureCounts(languageSampleFiles);
			
			ConstraintBuilder constraintBuilder = new ConstraintBuilder(pipe.getDataAlphabet(), pipe.getTargetAlphabet());
			ArrayList<GEConstraint> constraints = constraintBuilder.build(labelProportions, labeledFeatureCounts);
			
			CRFwithGE crfWithGE = new CRFwithGE();
			crfWithGE.printClassification(testingData, testingDataSequence, constraints);
		}
		else if(method.toLowerCase().equals("maxent-ge"))
		{
			WordClassification wordClassification = new WordClassification();
			Map<String, Double> labelProportions = wordClassification.getLanguageProportions(trainingData, testingData);
			Map<String, Map<String, Integer>> labeledFeatureCounts = trainingDataReader.labeledFeatureCounts(languageSampleFiles);
			
			ConstraintBuilder constraintBuilder = new ConstraintBuilder(pipe.getDataAlphabet(), pipe.getTargetAlphabet());
			ArrayList<MaxEntGEConstraint> constraints = constraintBuilder.buildForMaxEnt(labelProportions, labeledFeatureCounts);
			InstanceList unlabeledTestingData = removeLabels(testingData, pipe);
			
			MaxEntwithGE maxEntWithGE = new MaxEntwithGE();
			maxEntWithGE.printClassification(unlabeledTestingData, testingData, constraints);
		}
		else if(method.toLowerCase().equals("maxent"))
		{
			WordClassification wordClassification = new WordClassification();
			wordClassification.printClassification(trainingData, testingData);
		}
		else
		{
			System.out.println("Invalid method name: " + method);
			System.exit(0);
		}
	}
	
	public static InstanceList removeLabels(InstanceList il, Pipe pipe)
	{
		InstanceList ul = new InstanceList(new Noop());
		for(Instance i : il)
		{
			Instance j = new Instance(i.getData(), null, i.getName(), null);
			ul.add(j);
		}
		ul.setPipe(pipe);
		return ul;
	}
}
