package edu.umich.languageid;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import cc.mallet.classify.ClassifierTrainer;
import cc.mallet.classify.MaxEnt;
import cc.mallet.classify.MaxEntTrainer;
import cc.mallet.pipe.Noop;
import cc.mallet.types.Instance;
import cc.mallet.types.InstanceList;
import cc.mallet.types.Label;
import cc.mallet.types.Labeling;

import edu.umich.languageid.LabeledInstance;

public class MaxEntWrapper implements Classifier<Instance>
{
	MaxEnt classifier;
	InstanceList trainingData; // training data does not change over iterations
	
	public MaxEntWrapper(InstanceList training)
	{
		trainingData = training;
		ClassifierTrainer<MaxEnt> trainer = new MaxEntTrainer();
		trainer.train(training);
		
		classifier = trainer.getClassifier();
	}
	
	public MaxEntWrapper(InstanceList training, InstanceList testing)
	{
		trainingData = training;
		testing.addAll(training);
		ClassifierTrainer<MaxEnt> trainer = new MaxEntTrainer();
		trainer.train(testing);
		
		classifier = trainer.getClassifier();
	}
	
	public Map<String, Double> probabilities(Instance instance)
	{
		Map<String, Double> probDist = new HashMap<String, Double>();
		Labeling labeling = classifier.classify(instance).getLabeling();
		
		// assumes only two labels
		probDist.put(labeling.getLabelAtRank(0).toString(), labeling.getValueAtRank(1));
		probDist.put(labeling.getLabelAtRank(1).toString(), labeling.getValueAtRank(2));
		return probDist;
	}

	public double probability(Instance instance, String label)
	{
		Labeling labeling = classifier.classify(instance).getLabeling();
		
		// assumes only two labels
		if(labeling.getLabelAtRank(0).toString().equals(label))
			return labeling.getValueAtRank(0);
		return labeling.getValueAtRank(1);
	}

	public Classifier<Instance> createUpdatedClassifier(List<LabeledInstance<Instance>> hmmLabels)
	{
		Map<String, Label> labelsByString = new HashMap<String, Label>();
		for(Instance i : trainingData)
		{
			labelsByString.put(((Label)i.getTarget()).toString(), (Label)i.getLabeling());
		}
		
		
		InstanceList testing = new InstanceList(new Noop());
		for(LabeledInstance<Instance> labeledInstance : hmmLabels)
		{
			Instance i = new Instance(labeledInstance.getValue().getData(),	labelsByString.get(labeledInstance.getLabel()), 
					labeledInstance.getValue().getName(), labeledInstance.getValue().getSource());
			testing.add(i);
		}
		
		InstanceList il = trainingData.shallowClone();
		
		il.addAll(testing);
		ClassifierTrainer<MaxEnt> trainer = new MaxEntTrainer();
		trainer.train(il);
		
		classifier = trainer.getClassifier();
		
		return this;
	}

}
