package edu.umich.languageid;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;


public class TransitionProbabilityEstimator<T>
{

	public Map<String, Map<String, Double>> estimateTransitionProbabilities(List<LabeledInstance<T>> providedLabels, Set<String> allLabels)
	{
		Map<String, Map<String, Integer>> transitionCounts = new HashMap<String, Map<String, Integer>>();
		Map<String, Integer> labelOccurences = new HashMap<String, Integer>();
		
		String previousLabel = "";
		for(LabeledInstance<T> li : providedLabels)
		{
			if(previousLabel.length() == 0)
			{
				previousLabel = li.getLabel();
				continue;
			}
			
			if(transitionCounts.containsKey(previousLabel))
			{
				if(transitionCounts.get(previousLabel).containsKey(li.getLabel()))
					transitionCounts.get(previousLabel).put(li.getLabel(), transitionCounts.get(previousLabel).get(li.getLabel()) + 1);
				else
					transitionCounts.get(previousLabel).put(li.getLabel(), 1);
				
				labelOccurences.put(previousLabel, labelOccurences.get(previousLabel) + 1);
			}
			else
			{
				Map<String, Integer> map = new HashMap<String, Integer>();
				map.put(li.getLabel(), 1);
				transitionCounts.put(previousLabel, map);
				labelOccurences.put(previousLabel, 1);
			}
			
			previousLabel = li.getLabel();
		}
		
		Map<String, Map<String, Double>> transitionProbabilities = new HashMap<String, Map<String, Double>>();
		for(String fromLabel : allLabels)
		{
			for(String toLabel : allLabels)
			{
				double probability = 0;
				
				if(transitionCounts.containsKey(fromLabel) && transitionCounts.get(fromLabel).containsKey(toLabel))
					probability = (double) transitionCounts.get(fromLabel).get(toLabel) / labelOccurences.get(fromLabel);
				if(transitionProbabilities.containsKey(fromLabel))
				{
					transitionProbabilities.get(fromLabel).put(toLabel, probability);
				}
				else
				{
					Map<String, Double> map = new HashMap<String, Double>();
					map.put(toLabel, probability);
					transitionProbabilities.put(fromLabel, map);
				}
			}
		}
		
		for(String label : labelOccurences.keySet())
		{
			System.out.println(label + " " + labelOccurences.get(label));
		}
		
		return transitionProbabilities;
	}
	
}
