package edu.umich.languageid;

import java.util.HashMap;
import java.util.Map;

import cc.mallet.classify.C45;
import cc.mallet.classify.C45Trainer;
import cc.mallet.classify.ClassifierTrainer;
import cc.mallet.classify.DecisionTree;
import cc.mallet.classify.DecisionTreeTrainer;
import cc.mallet.classify.MaxEnt;
import cc.mallet.classify.MaxEntTrainer;
import cc.mallet.classify.NaiveBayes;
import cc.mallet.classify.NaiveBayesTrainer;
import cc.mallet.classify.Trial;
import cc.mallet.classify.Winnow;
import cc.mallet.classify.WinnowTrainer;
import cc.mallet.types.Instance;
import cc.mallet.types.InstanceList;
import cc.mallet.types.Labeling;

public class WordClassification
{
	public double naiveBayesAccuracy(InstanceList trainingData, InstanceList testingData)
	{
		ClassifierTrainer<NaiveBayes> trainer = new NaiveBayesTrainer();
		trainer.train(trainingData);
		
		NaiveBayes classifier = trainer.getClassifier();
		Trial test = new Trial(classifier, testingData);
		return test.getAccuracy();
	}
	
	public double logisticRegressionAccuracy(InstanceList trainingData, InstanceList testingData)
	{
		ClassifierTrainer<MaxEnt> trainer = new MaxEntTrainer();
		trainer.train(trainingData);
		
		MaxEnt classifier = trainer.getClassifier();
		Trial test = new Trial(classifier, testingData);
		return test.getAccuracy();
	}
	
	public double decisionTreeAccuracy(InstanceList trainingData, InstanceList testingData)
	{
		ClassifierTrainer<DecisionTree> trainer = new DecisionTreeTrainer();
		trainer.train(trainingData);
		
		DecisionTree classifier = trainer.getClassifier();
		Trial test = new Trial(classifier, testingData);
		return test.getAccuracy();
	}
	
	public double c45Accuracy(InstanceList trainingData, InstanceList testingData)
	{
		ClassifierTrainer<C45> trainer = new C45Trainer();
		trainer.train(trainingData);
		
		C45 classifier = trainer.getClassifier();
		Trial test = new Trial(classifier, testingData);
		return test.getAccuracy();
	}
	
	public double winnowAccuracy(InstanceList trainingData, InstanceList testingData)
	{
		ClassifierTrainer<Winnow> trainer = new WinnowTrainer();
		trainer.train(trainingData);
		
		Winnow classifier = trainer.getClassifier();
		Trial test = new Trial(classifier, testingData);
		return test.getAccuracy();
	}
	
	public void printClassification(InstanceList trainingData, InstanceList testingData)
	{
		ClassifierTrainer<MaxEnt> trainer = new MaxEntTrainer();
		trainer.train(trainingData);
		
		MaxEnt classifier = trainer.getClassifier();
		String previousClass = "";
		for(Instance i : testingData)
		{
			String bestClass = classifier.classify(i).getLabeling().getBestLabel().toString();
			if(bestClass.equals(previousClass))
			{
				System.out.print(" " + i.getName());
			}
			else
			{
				System.out.println();
				System.out.print("[[" + bestClass + "]] " + i.getName());
			}
			previousClass = bestClass;
		}
	}
	
	public Map<String, Double> getLanguageProportions(InstanceList trainingData, InstanceList testingData)
	{		
		ClassifierTrainer<MaxEnt> trainer = new MaxEntTrainer();
		trainer.train(trainingData);
		
		Map<String, Integer> labelCounts = new HashMap<String, Integer>();
		
		MaxEnt classifier = trainer.getClassifier();
		for(Instance i : testingData)
		{
			String bestClass = classifier.classify(i).getLabeling().getBestLabel().toString();
			
			if(labelCounts.containsKey(bestClass))
				labelCounts.put(bestClass, labelCounts.get(bestClass)+1);
			else
				labelCounts.put(bestClass, 1);
		}
		
		Map<String, Double> proportions = new HashMap<String, Double>();
		for(String label :labelCounts.keySet())
		{
			proportions.put(label, (double)labelCounts.get(label)/testingData.size());
		}
		
		return proportions;
	}
	
	public Map<String, Double> getDocumentLanguageGuessDistribution(InstanceList trainingData, Instance testingData)
	{
		ClassifierTrainer<MaxEnt> trainer = new MaxEntTrainer();
		trainer.train(trainingData);
		
		Map<String, Integer> labelCounts = new HashMap<String, Integer>();
		
		MaxEnt classifier = trainer.getClassifier();
		Labeling labeling = classifier.classify(testingData).getLabeling();
		
		Map<String, Double> distribution = new HashMap<String, Double>();
		for(int i=0; i<labeling.numLocations(); ++i)
		{
			distribution.put(labeling.labelAtLocation(i).toString(), labeling.valueAtLocation(i));
		}
		
		return distribution;
	}
	
	public String getDocumentLanguageBestGuess(InstanceList trainingData, Instance testingData)
	{
		ClassifierTrainer<NaiveBayes> trainer = new NaiveBayesTrainer();
		trainer.train(trainingData);
		
		NaiveBayes classifier = trainer.getClassifier();
		return classifier.classify(testingData).getLabeling().getBestLabel().toString();
	}
}

