package edu.umich.languageid.io;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import cc.mallet.pipe.Noop;
import cc.mallet.pipe.Pipe;
import cc.mallet.types.FeatureVector;
import cc.mallet.types.FeatureVectorSequence;
import cc.mallet.types.Instance;
import cc.mallet.types.InstanceList;
import cc.mallet.types.Label;
import cc.mallet.types.LabelAlphabet;
import cc.mallet.types.LabelSequence;
import edu.umich.languageid.FeatureConfiguration;

public class TestingDataReader
{
	FeatureConfiguration features;
	Pipe pipe;
	
	int numTokens = 0;
	
	public TestingDataReader()
	{
		this(new FeatureConfiguration(), new Noop());
	}
	
	public TestingDataReader(FeatureConfiguration features, Pipe pipe)
	{
		this.features = features;
		this.pipe = pipe;
	}
	
	public Map<String, Integer> languageWordCounts(File mixedLangDoc)
	{
		Map<String, Integer> languageWordCounts = new HashMap<String, Integer>();
				
		try
		{
			String fileContents = readFile(mixedLangDoc);
			WordInContextReader wicr = new WordInContextReader(false);
			
			List<WordInContext> wics = wicr.getWordsInContext(fileContents);
						
			for(WordInContext wic : wics)
			{
				String language = wic.getLabel();
				if(!languageWordCounts.containsKey(language))
					languageWordCounts.put(language, 0);
				languageWordCounts.put(language, languageWordCounts.get(language)+1);
			}
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return languageWordCounts;
	}
	
	public Map<String, Map<String, Integer>> labeledFeatureCounts(File mixedLangDoc, String lang1Label, String lang2Label)
	{
		String[] labels = new String[2];
		labels[0] = lang1Label;
		labels[1] = lang2Label;
		numTokens = 0;
		
		Map<String, List<List<String>>> allFeatureInstances = new HashMap<String, List<List<String>>>();
		WordInstanceBuilder wordToInstance = new WordInstanceBuilder(features, pipe.getDataAlphabet());
		
		try
		{
			String fileContents = readFile(mixedLangDoc);
			WordInContextReader wicr = new WordInContextReader(false);
			
			List<WordInContext> wics = wicr.getWordsInContext(fileContents);
			
			for(WordInContext wic : wics)
			{
				List<String> wordFeatures = wordToInstance.featureList(wic.getWord());
				if(!allFeatureInstances.containsKey(wic.getLabel()))
					allFeatureInstances.put(wic.getLabel(), new ArrayList<List<String>>());
				allFeatureInstances.get(wic.getLabel()).add(wordFeatures);
				++numTokens;
			}
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		Map<String, Map<String, Integer>> sampledFeatureCounts = new HashMap<String, Map<String, Integer>>();
		
		for(String label : allFeatureInstances.keySet())
		{
			for(List<String> features : allFeatureInstances.get(label))
			{
				for(String feature : features)
				{
					if(!sampledFeatureCounts.containsKey(feature))
						sampledFeatureCounts.put(feature, new HashMap<String, Integer>());
					if(sampledFeatureCounts.get(feature).containsKey(label))
						sampledFeatureCounts.get(feature).put(label, sampledFeatureCounts.get(feature).get(label) + 1);
					else
						sampledFeatureCounts.get(feature).put(label, 1);
				}
			}
		}
		
		return sampledFeatureCounts;
	}
	
	public Map<String, Map<String, Integer>> labeledWordCounts(File mixedLangDoc)
	{
		Map<String, List<String>> allWordInstances = new HashMap<String, List<String>>();
				
		try
		{
			String fileContents = readFile(mixedLangDoc);
			WordInContextReader wicr = new WordInContextReader(false);
			
			List<WordInContext> wics = wicr.getWordsInContext(fileContents);
			
			for(WordInContext wic : wics)
			{
				String type = wic.getWord();
				if(!allWordInstances.containsKey(wic.getLabel()))
					allWordInstances.put(wic.getLabel(), new ArrayList<String>());
				allWordInstances.get(wic.getLabel()).add(type);
				++numTokens;
			}
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		Map<String, Map<String, Integer>> sampledWordCounts = new HashMap<String, Map<String, Integer>>();
		
		for(String label : allWordInstances.keySet())
		{
			for(String type : allWordInstances.get(label))
			{
				if(!sampledWordCounts.containsKey(type))
					sampledWordCounts.put(type, new HashMap<String, Integer>());
				if(sampledWordCounts.get(type).containsKey(label))
					sampledWordCounts.get(type).put(label, sampledWordCounts.get(type).get(label) + 1);
				else
					sampledWordCounts.get(type).put(label, 1);
				
			}
		}
		
		return sampledWordCounts;
	}
	
	public InstanceList readSequence(File mixedLangDoc)
	{
		numTokens = 0;
		
		WordInstanceBuilder wordToInstance = new WordInstanceBuilder(features, pipe.getDataAlphabet());
		FeatureVectorSequence fvs = null;
		LabelSequence tokenLabels = null;
				
		try
		{
			String fileContents = readFile(mixedLangDoc);
			WordInContextReader wicr = new WordInContextReader(false);
			
			List<WordInContext> wics = wicr.getWordsInContext(fileContents);
			FeatureVector[] featureVectors = new FeatureVector[wics.size()];
			tokenLabels = new LabelSequence((LabelAlphabet)pipe.getTargetAlphabet(), wics.size());
			
			for(WordInContext wic : wics)
			{
				FeatureVector fv = wordToInstance.convertWordToFeatureVector(wic);
				featureVectors[numTokens] = fv;
				tokenLabels.add(wic.getLabel());
				++numTokens;
			}
			
			fvs = new FeatureVectorSequence(featureVectors);
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		Instance i = new Instance(fvs, tokenLabels, "alldata", null);
		InstanceList iList = new InstanceList(new Noop());
		iList.addThruPipe(i);
		return iList;
	}
	
	public InstanceList readTestingData(File mixedLangDoc)
	{
		//Alphabet dataAlphabet = new Alphabet(String.class);
		//Alphabet targetAlphabet = new Alphabet(labels);
		InstanceList instanceList = new InstanceList(pipe);
		WordInstanceBuilder wordToInstance = new WordInstanceBuilder(features, pipe.getDataAlphabet());
		
		try
		{
			String fileContents = readFile(mixedLangDoc);
			WordInContextReader wicr = new WordInContextReader(false);
			
			for(WordInContext wic : wicr.getWordsInContext(fileContents))
			{
				++numTokens;
				Instance i = wordToInstance.convertWordToInstance(wic);
				instanceList.addThruPipe(i);
			}
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return instanceList;
	}
	
	public Set<String> allObservedTypes(File mixedLangDoc)
	{
		Set<String> tokens = new HashSet<String>();
		
		try
		{
			String fileContents = readFile(mixedLangDoc);
			WordInContextReader wicr = new WordInContextReader(false);
			
			for(WordInContext wic : wicr.getWordsInContext(fileContents))
			{
				tokens.add(wic.getWord());
			}
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return tokens;
	}
	
	private static String readFile(File path) throws IOException {
		  FileInputStream stream = new FileInputStream(path);
		  try {
		    FileChannel fc = stream.getChannel();
		    MappedByteBuffer bb = fc.map(FileChannel.MapMode.READ_ONLY, 0, fc.size());
		    /* Instead of using default, pass in a decoder. */
		    return Charset.defaultCharset().decode(bb).toString();
		  }
		  finally {
		    stream.close();
		  }
		}
	
	public int getNumTokens()
	{
		return numTokens;
	}

	public Map<String, Integer> createMockDocument(File file, int docSize)
	{
		Map<String, Integer> sampledFeatureCounts = new HashMap<String, Integer>();
		List<List<String>> allFeatureInstances = new ArrayList<List<String>>();
		WordInstanceBuilder wordToInstance = new WordInstanceBuilder(features, pipe.getDataAlphabet());
		try
		{
			BufferedReader reader = new BufferedReader(new FileReader(file));
			String line = "";
			while((line = reader.readLine()) != null)
			{
				String[] tokens = line.split("[\\s\\p{Punct}\\d]+");
				for(String token : tokens)
				{
					if(token.length() == 0)
						continue;
					List<String> wordFeatures = wordToInstance.featureList(token);
					
					allFeatureInstances.add(wordFeatures);
				}
			}
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
		}
		
		Random r = new Random();
		for(int k=0; k<docSize; ++k)
		{
			for(String feature : allFeatureInstances.get(r.nextInt(allFeatureInstances.size())))
			{
				if(sampledFeatureCounts.containsKey(feature))
					sampledFeatureCounts.put(feature, sampledFeatureCounts.get(feature) + 1);
				else
					sampledFeatureCounts.put(feature, 1);
			}
		}
		
		return sampledFeatureCounts;
	}
	
	public Instance toInstance(Map<String, Integer> featureCounts, String trueLabel)
	{
		String data = "";
		for(String featureName : featureCounts.keySet())
		{
			for(int k=0; k<featureCounts.get(featureName); ++k)
			{
				data += featureName + " ";
			}
		}
		Instance i = new Instance(data, trueLabel, "mockDocument_" + trueLabel, null);
		return pipe.instanceFrom(i);
	}
	
	public Instance toUnlabeledInstance(Map<String, Integer> featureCounts, String trueLabel)
	{
		String data = "";
		for(String featureName : featureCounts.keySet())
		{
			for(int k=0; k<featureCounts.get(featureName); ++k)
			{
				data += featureName + " ";
			}
		}
		Instance i = new Instance(data, null, "mockDocument_" + trueLabel, null);
		return pipe.instanceFrom(i);
	}

	public InstanceList toInstanceList(Instance mockDocument)
	{
		InstanceList ul = new InstanceList(new Noop());
		ul.add(mockDocument);
		return ul;
	}
}
